/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* bos530 src/bos/kernel/sys/aacct.h 1.30                                 */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2003,2004          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)75       1.30  src/bos/kernel/sys/aacct.h, sysproc, bos530 5/14/04 15:34:58 */
#ifndef _H_AACCT
#define _H_AACCT

#include <sys/types.h>
#include <sys/proc.h>		/* struct proc_counts */

/*
 * The projid_t has three components: a type, reserved flags, and an id.
 * The type is the upper nibble and is read-only.  The flags are the next
 * nibble and are also read-only.  The id is the lower 24 bits, resulting
 * in valid project ids in the range 0x0 - 0x00FFFFFF.  Actually, project
 * id 0 is reserved, so the first project ID that can be added is 1.
 */
#define PROJIDMASK      0x00FFFFFF
#define PROJTYPEMASK    0xF0000000
#define PROJIDUIDREL    0x10000000
#define PROJIDGIDREL    0x20000000
#define PROJORIGMASK    0x0F000000
#define PROJORIGSHIFT   24
#define PROJTYPESHIFT   28
#define PROJIDMAX       PROJIDMASK
#define PROJIDMINU	1		/* Minimum User Defined Project */
#define PROJIDMIN      	0 

#define PROJID(id)      ((id) & PROJIDMASK)
#define PROJORIG(id)    (((id) & PROJORIGMASK) >> PROJORIGSHIFT)
#define PROJTYPE(id)    (((id) & PROJTYPEMASK) >> PROJTYPESHIFT)

/* default project */
#define SYSPROJNAME 	"System"
#define SYSPROJNAMELEN 	7
#define SYSPROJID	0

/*
 * Project related definitions
 */
#define PROJNAMELEN     32
typedef int projid_t;

#define PROJFLAGS_AGGR	0x1

struct project {
        char     name[PROJNAMELEN];
        projid_t id;
        int      flags;
};

/*
 * These APIs updated the specified project repository and the kernel, 
 * if necessary.  The kernel is updated, if the project repository 
 * has been loaded into the kernel.
 */
int projdballoc(void **handle);
int projdbfree(void *handle);
int projdbfinit(void *handle, char *file, int mode);
int addprojdb(void *handle, struct project *proj, char *comment);
int rmprojdb(void *handle, struct project *proj, int flag);
int getfirstprojdb(void *handle, struct project *proj, char *comm);
int getnextprojdb(void *handle, struct project *proj, char *comm);
int getprojdb(void *handle, struct project *proj, int flag);
int chprojattrdb(void *handle, struct project *proj, int cmd);

/* flag parameter values for rmproj(), getproj(), rmprojdb() and getprojdb() */
#define PROJ_NAME 		1
#define PROJ_NUM 		2

/* cmd parameter values for chprojattrdb() and chprojattr() */
#define PROJ_ENABLE_AGGR 	1
#define PROJ_DISABLE_AGGR 	0

/*
 * These APIs are intended for high level job schedulers that wish to
 * assign their own project codes.  These APIs do not update the project 
 * repository.  They are designed to interface directly with the kernel, 
 * so project assignments may be performed without having to explicitly 
 * load the project repository.  Since these APIs bypass the registry, 
 * applications that post process accounting data and which rely on the 
 * registry to associate project names with project numbers may not 
 * function properly.  However, they should be able to display the 
 * project number. 
 */
int addproj(struct project *proj);
int chprojattr(struct project *proj, int cmd);
int rmproj(struct project *proj, int flag);
int getproj(struct project *proj, int flag);
int getprojs(struct project *proj, int * nelems);
int proj_execve(char *path, char *const arg[], 
		char *const env[], projid_t proj, int force);
int getsubproj(unsigned long long *subproj);
int setsubproj(unsigned long long *subproj);
projid_t getmyproj(void);
int setmyproj(projid_t id);

/*
 * The tusage structure is for use by ARM transactions, and contains the
 * usage statistics these transactions can report.  It can optionally be
 * passed to acct_put for inclusion in an accounting transaction record.
 */
struct tusage {
	unsigned long long tu_cpu;	/* cpu used, in u-secs */
	unsigned long long tu_res1;
};

#define TUSAGE_ZERO(TU) {			\
		(TU).tu_cpu  = 0;		\
		(TU).tu_res1 = 0; }

#define TUSAGE_ADD(TU1, TU2) {			\
		(TU1).tu_cpu  += (TU2).tu_cpu;	\
		(TU1).tu_res1 += (TU2).tu_res1; }

#define TUSAGE_SUB(TU1, TU2) {			\
		(TU1).tu_cpu  -= (TU2).tu_cpu;	\
		(TU1).tu_res1 -= (TU2).tu_res1; }

#ifdef _KERNEL

/* get cpu usage of current thread. */
u_longlong_t acct_get_usage(struct tusage *);	

/*
 * The kernel exports aacct_flags to indicate if accounting
 * is enabled overall.
 *
 * Knowledge of which individual transactions are enabled is passed
 * to registered interval callout routines.
 *
 * Note that the value of aacct_flags is returned by the ACC_QUERY_STATE
 * option of acctctl, so the flag definitions are made visible outside the
 * kernel even though the flag word itself is not.
 */

extern int aacct_flags;
#endif


/*
 * These flags identify global accounting state
 */ 
#define AACCT_ON	0x00000001	/* accounting enabled */
#define AACCT_LOADED	0x00000002	/* kernel extension loaded */
#define AACCT_SYS_INT	0x00000004	/* system interval is active */
#define AACCT_PROC_INT	0x00000008	/* process interval is active */
#define AACCT_AGGR_PROC	0x00000010	/* process aggregation is enabled */
#define AACCT_AGGR_KE	0x00000020	/* kernext aggregation is enabled */
#define AACCT_AGGR_ARM	0x00000040	/* ARM aggregation is enabled */

#ifdef _KERNEL

#define ACCT_ENABLED	(aacct_flags & AACCT_ON)

/*
 * The acct_put routine is exported from the kernel, so it can always be
 * called whether advanced accounting is active or loaded or not.
 *
 * This service builds the common header from its parameters and values
 * in curproc, appends the specific transaction data to create an accounting
 * transaction record, and schedules it to be written to an accounting file.
 *
 * The projid may be	specified directly if known,
 *			set to PROJID_SYSTEM for system interval transactions
 *			set to PROJID_UNKNOWN to request curproc's project ID
 *
 * If tusage is non-NULL, a common subheader will be generated.
 *
 * The possible flag #defines are below.
 */
void acct_put(	int trid,		/* transaction ID */
		int flags,		/* flags, eg aggregation */
		int projid,		/* project ID */
		struct tusage * usage,	/* transaction usage values */
		void *trdata,		/* transaction-specific data */
		int tr_len);		/* size of trdata */

#define	PROJID_SYSTEM	0	/* projid value for system interval trans's */
#define PROJID_NOACCT	-1	/* projid value for no accounting */
#define PROJID_UNKNOWN	-2	/* get projid value from curproc */

/*
 * The follow flag value(s) are for the public interface to acct_put,
 * and must all be defined in the right half of the integer.
 * The left half of the integer is reserved for internal use.
 */
#define	ACCT_PUT_DIRECT	0x00000001	/* bypass aggregation */

/*
 * Kernel services to register/unregister handlers for interval accounting.
 *
 * Only up to 15 characters will be kept from the transname parameter.
 */
int acct_interval_register(	int transid,
				int cmds,
				int (*handler)(int transid, int cmd, void *arg),
				void *arg,
				unsigned long *h_token,
				char *transname);

/*
 * These are the possible "cmds" that will trigger callout to the registered
 * handler.  If new cmds need to be implemented, be sure to
 *
 * 1. include them in ACCT_CMD_ALL
 * 2. assign consecutive bits from the right (low order).
 *
 * The upper half of the cmds integer is reserved for internal system use.
 */
#define ACCT_CMD_ENABLE		1
#define ACCT_CMD_DISABLE	2
#define ACCT_CMD_INTERVAL	4
#define ACCT_CMD_FSWITCH	8

#define ACCT_CMD_ALL	(ACCT_CMD_ENABLE | ACCT_CMD_DISABLE | \
		       ACCT_CMD_INTERVAL | ACCT_CMD_FSWITCH)

int acct_interval_unregister(unsigned long h_token);

/*
 * Special atomic increment and clear services are provided to allow
 * machine independent updating of unsigned long long values.
 * However, only 63 bit values are guaranteed.
 *
 * 64bit kernel - can use ldarx/stdcx in the natural way, providing a
 *		very efficient 64bit service.
 *
 * 32bit kernel on 64bit hardware - can use ldarx/stdcx in a disabled
 *		section to prevent interrupts, providing a moderately
 *		efficient 64bit service.
 *
 * 32bit kernel on 32bit hardware - must use the high-order bit of the
 *		doubleword as an interlock with sync and isync, providing
 *		a pretty inefficient 63bit service.
 *
 * All services return the previous value of the unsigned long long.
 * Or as many bits as we can depending on the platform involved.
 * The increment services will only perform an incrment if accounting
 * is enabled, since this is a potentially expensive operation.
 *
 * The acct_add_LL and acct_zero_LL services are intended for kernel
 * extensions maintaining their own system data whose addresses they know.
 */
unsigned long long acct_add_LL(unsigned long long *, unsigned int);
unsigned long long acct_zero_LL(unsigned long long *);

/*
 * The acct_add_proc service is intended for kernel extension extensions
 * that need to update the non-sysproc process-based counters in the
 * proc structure (which they are not allowed to address directly).
 * The indices to use are #defined below with the tran_proc structure.
 * For example, to update the local I/O count, use the TRP_LOCAL_IO index.
 */
unsigned long long acct_add_proc(unsigned int counter_index, unsigned int incr);

/* Get calling process' project id */
projid_t acct_get_projid(void);

/* Get project id for the given attributes */
projid_t acct_classify(char *appname, uid_t uid, gid_t gid);

#endif	/* _KERNEL */

/*
 * Kernel transaction IDs are defined here, and implicitly mapped to
 * their purpose by choosing a meaningful name for the #define value.
 * This name should be of the form TRID_purpose, where "purpose" is
 * chosen to identify the type of transaction.
 *
 * Also add the "purpose" as an array assignment in the ACCT_NAME macro.
 *
 * When a transaction ID is defined here, the corresponding structure
 * mapping that transaction's specific data must also be defined in the
 * next section of this header file.  The structure name should be
 * tran_purpose, where "purpose" would be identical to the corresponding
 * "purpose" in the transaction ID name.
 */
#define	TRID_pad	0	/* reserved for file padding */
#define	TRID_proc	1	/* process (non-aggregated) transaction */
#define TRID_agg_proc	2	/* aggregated process transaction */
#define TRID_agg_app	3	/* agregated application use */
#define TRID_cpumem	4	/* system configuration and activity */
#define	TRID_policy	5	/* policy change */
#define	TRID_file	6	/* file system activity */
#define TRID_netif	7	/* network interface activity */
#define TRID_disk	8	/* physical disk volume activity */
#define TRID_lostdata	9	/* cpu lost data record */
#define TRID_vscsi_target 10    /* virtual scsi target adapter */
#define TRID_vscsi_client 11    /* virtual scsi client adapter */
#define	TRID_agg_ke	  12	/* 3rd party common aggregation record */
#define	TRID_arm_applenv  13	/* ARM application instance record */
#define	TRID_arm_trenv	  14	/* ARM transaction definition record */
#define	TRID_arm_tran	  15	/* ARM transaction instance record */
#define	TRID_agg_arm      16	/* ARM aggregate transaction record */
#define TRID_project	  17	/* project definitions */

#define TRID_KERN_MAX	127	/* highest number for base kernel trans */
#define TRID_MAX	255	/* highest allowed transaction ID */

/*
 * Each accounting file will contain a simple header in the first page
 * of the file, with nothing else in this page.  Normal accounting records 
 * will start in the second page.  The header is simply:
 */
struct acct_file_header {
	unsigned int afh_version;		/* just in case... */
	unsigned int afh_flags;			/* ... */
	unsigned long long afh_offset;		/* current file pointer */
	unsigned long long afh_file_size;	/* amount of space in file */
	unsigned int afh_state;			/* for debugging only */
	unsigned int afh_id;			/* for debugging only */
	unsigned long long afh_time_first;	/* open date, in seconds */
	unsigned long long afh_time_last;	/* close date, maybe */
	char afh_system_id[32];			/* uname -u */
	char afh_system_model[32];		/* uname -M */
	char afh_host_name[32];			/* uname -n */
	char afh_partition_name[256];		/* uname -L - Name */
	char afh_partition_number[4];		/* uname -L,  -1 if not LPAR */
	char afh_PAD[4];
	unsigned long long afh_reserved[461];	/* pad to full page */
};

/*
 * General format of an accounting transaction record.
 *
 * Variable length data specific to the record begins at tran_data,
 * which may optionally include a subheader.  The first integer of
 * this subheader will contain the subheader's total size.
 * The size of the transaction-specific data at tran_data (not
 * including any subheader) is implied by the tran_id.
 *
 * Note - the size of this structure may be misleading!
 * Use TRAN_HDR_SIZE, defined below.
 */
struct aacct_tran_header {
	unsigned int		tran_size;	/* size of entire record */
	unsigned int		tran_id;	/* transaction ID */
	unsigned char		tran_flags;	/* transaction flags (below) */
	unsigned char		tran_pad1[3];
	int			tran_project;	/* project ID */
	unsigned long long	tran_subproj;	/* subproject ID (for LL) */
	unsigned long long	tran_time;	/* time record built, usecs */
	char			tran_data[4];	/* variable length data */
};

/*
 * Flags defined in tran_flags
 */
#define	TRAN_KERNEL	0x01	/* kernel transaction gen'd by acct_put */
#define TRAN_SUBHEADER	0x02	/* tran_data starts off with a subheader */
#define TRAN_PROJ_RUSER	0x04	/* proj ID generated relative to UID */
#define TRAN_PROJ_RGRP	0x08	/* proj ID generated relative to GID */

/*
 * Size of the common header == offset to the variable transaction data.
 */
#define TRAN_HDR_SIZE	(long)&(((struct aacct_tran_header *)0)->tran_data)

/******** The structures here map transaction specific data *************/

/*
 * There are a lot of doubleword counters that exist identically in
 * the proc structure (collecting data) and in the process record
 * (reporting it).  The trp_counts array in struct proc_counts allows us
 * to add counters in one place, and have them appear in both the proc and
 * process accounting record structures (discrete and aggregated) identically.
 * The array organization lets the data be copied by exit and interval
 * accounting without knowing the names of the individual fields.
 * The #defines let you reference the fields as if they were simple
 * long longs, as opposed to via explicit subscripting, to increment them.
 * To add a new accounting field, all you have to do is add two #defines,
 * (uppercase for the new data's index, lowercase for its scalar name)
 * increment TRP_NUM_COUNTS in proc.h, increment TRP_MAX and possibly
 * TRP_MAX_PROC below, and add the actual subsystem code to increment it.
 * Everything else will be taken care of automatically.
 *
 * The first TRP_MAX_PROC+1 elements are protected by the pv_lock_d,
 * and updated in process_proc_table, at interval end, or in exit.
 * The first TRP_LAST_TB+1 elements are collected in timebase units in
 * the proc structure, but converted to microseconds when moved to the
 * accounting transaction.
 *
 * The remaining elements are updated atomically outside sysproc,
 * and are copied/cleared atomically by interval accounting.
 * The code that increments them must use acct_add_LL, or, from a kernel
 * extension, acct_add_proc using the index number defined here.
 *
 * WARNING WARNING WARNING - New fields may be added compatibly only at
 * the end of the process record.  In effect, this means that TRP_LAST_TB
 * and TRP_MAX_PROC should never be changed.
 * TRP_MAX is the only value that may be compatibly increased.
 * This means that it is easy to add non-sysproc counters.  Adding a new
 * sysproc counter at the end will require source code in acct_proc()
 * to copy it while the pv_lock_d is still held.
 *
 * See the immediately following array reference definitions for comments
 * on the individual counters.
 */
#define TRP_ELAPSED	0
#define TRP_THREADTIME	1
#define TRP_CPUTIME	2
#define TRP_LAST_TB		2	/* highest timebase value */

#define TRP_DPAGE_SEC	3
#define TRP_RPAGE_SEC	4
#define TRP_VPAGE_SEC	5
#define TRP_MAX_PROC		5	/* highest index for sysproc data */

#define TRP_LOCAL_IO	6
#define TRP_OTHER_IO	7
#define TRP_LSOCK_IO	8
#define TRP_RSOCK_IO	9
#define TRP_MAX			9	/* highest index for all this data */

/*
 * Note: while the first three time statistics are reported to the user
 * via accounting transactions in usecs, the values are maintained in TB
 * units in the proc structure for efficiency reasons.
 */
#define	trp_elapsed	trp_count[0]	/* proc elapsed u-secs */
#define	trp_threadtime	trp_count[1]	/* combined threads elapsed u-secs */
#define	trp_cputime	trp_count[2]	/* proc (combined threads) cpu u-secs */
#define	trp_dpage_sec	trp_count[3]	/* page secs of disk pages */
#define	trp_rpage_sec	trp_count[4]	/* page secs of real pages */
#define	trp_vpage_sec	trp_count[5]	/* page secs of virt mem */
#define	trp_local_io	trp_count[6]	/* bytes file i/o to jfs, j2, etc. */
#define	trp_other_io	trp_count[7]	/* bytes file i/o to nfs, dfs, etc. */
#define	trp_lsock_io	trp_count[8]	/* bytes unix-domain/loopback sockets */
#define	trp_rsock_io	trp_count[9]	/* bytes over remote sockets */

/*
 * The proc_counts structure is actually defined in proc.h, but shown
 * here to indicate its nature.  The structure is used not only in the
 * proc structure, where data is collected, but in the various process
 * level accounting transactions as well.
 *
 *	struct proc_counts {
 *		unsigned long long	trp_count[TRP_NUM_COUNTS];
 *	} p_acct;	
 *
 * Both headers must be consistent, as checked below.
 */
struct trp_max_check {
	char	a[TRP_MAX < TRP_NUM_COUNTS ? 1 : -1];
};

struct tran_proc {
	int			trp_version;
	uid_t			trp_uid;	/* user id */
	gid_t			trp_gid;	/* group id */
	pid32_t			trp_pid;	/* process id */
	int			trp_service_cl;	/* eWLM service class */
	int			trp_flags;	/* flags, see below */
	char			trp_comm[12];	/* base command name */
	int			trp_pad[3];
	dev64_t			trp_tty;	/* controlling terminal */
						/* time 0 is the epoch below */
	unsigned long long	trp_start;	/* proc start time in secs */
	unsigned long long	trp_wlmkey;	/* WLM class key */
	struct proc_counts	trp_counts;	/* incrementing statistics */
};

#define TRP_FINAL	0x01	/* This is the final exit for the process */
#define TRP_CORE	0x02	/* process created a core file */
#define TRP_KILLED	0x04	/* process was killed by a signal */
#define TRP_KPROC	0x08	/* kernel process */
#define TRP_CHKPNT	0x10	/* terminated because of checkpoint */

/**********************************************************************/

struct tran_agg_proc {
	int			trp_version;
	int			trp_pad;
	unsigned long long	trp_start;	/* start time in seconds */
	uid_t			trp_uid;	/* user id */
	unsigned int		trp_procs;	/* number of procs aggregated */
	struct proc_counts	trp_counts;	/* incrementing statistics */
};

struct tran_agg_app {
	int			trp_version;
	int			trp_pad;
	unsigned long long	trp_start;	/* start time in seconds */
	uid_t			trp_uid;	/* user id */
	unsigned int		trp_commands;	/* invocations aggregated */
	dev64_t			trp_dev;	/* devno for command */
	ino64_t			trp_ino;	/* inode for command */
	struct proc_counts	trp_counts;	/* incrementing statistics */
};

struct tran_agg_ke {
	int			trp_version;
	int			trp_pad;
	unsigned long long	trp_start;	/* start time in seconds */
	unsigned int		trp_trid;	/* transaction id */
	unsigned int		trp_count;	/* # transactions aggregated */
	char			trp_comm[12];	/* KE name from U_comm */
	struct tusage		trp_usage;	/* resource utilization */
};

/**********************************************************************/


struct tran_cpumem {
        unsigned int rec_type;

#define ACCT_CPU_ADD_PRE        0
#define ACCT_CPU_ADD_POST       1
#define ACCT_CPU_REMOVE_PRE     2
#define ACCT_CPU_REMOVE_POST    3
#define ACCT_MEM_ADD_PRE        4
#define ACCT_MEM_ADD_POST       5
#define ACCT_MEM_REMOVE_PRE     6
#define ACCT_MEM_REMOVE_POST    7
#define ACCT_ENTCAP_ADD_PRE     8
#define ACCT_ENTCAP_ADD_POST    9
#define ACCT_ENTCAP_REMOVE_PRE  10
#define ACCT_ENTCAP_REMOVE_POST 11
#define ACCT_SYSTEM_INTERVAL    12
#define ACCT_SYSTEM_INT_ENABLE  13
#define ACCT_SYSTEM_INT_DISABLE 14
#define ACCT_LGPG_POOL_INCREASE 15
#define ACCT_LGPG_POOL_DECREASE 16

        unsigned int ncpus;
        unsigned int entitled_cap;
        unsigned int pad;
        unsigned long long idle_time;
        unsigned long long iowait_time;
        unsigned long long sprocess_time;
        unsigned long long uprocess_time;
        unsigned long long interrupt_time;
        unsigned long long memory_size;
        unsigned long long lgpg_total;
        unsigned long long lgpg_inuse;
        unsigned int pgsp_ins;
        unsigned int pgsp_outs;
        unsigned int num_ios;
        unsigned int page_steals;
}; 

/*
 * Project definition record.  Written when projects are loaded.
 *
 * The project record is variable length and consists of a header
 * (tran_project), followed by an array of projects (proj_rec),
 * followed by a block of strings containing the project names.
 * Each proj_rec contains an offset to the associated project name,
 * relative to the start of the proj_rec array.
 */
struct proj_rec {
    unsigned	projid;
    ushort_t	origin;
    ushort_t	offset;
};

#define ACCT_ORIGIN_LOCAL	1

struct tran_project {
    int	resvd[2];
    unsigned nproj;
    unsigned size;
};

/*
 * Policy record.  Written when a policy is loaded or unloaded.
 */
struct tran_policy {
    int		resvd[2];
    unsigned	type;
    unsigned	event;
}; 

/* policy types */
#define ACCT_POL_ADM	1
#define ACCT_POL_USR	2
#define ACCT_POL_GRP	3

/* policy events */
#define ACCT_EVNT_LOAD		1
#define ACCT_EVNT_UNLOAD	2

/**********************************************************************/

/* advanced accounting file system data for system level statistic
 *
 * Note: The following struct tran_filesystem contains
 *       the top portion of a file system transaction record,
 *       which is the main portion of the record and has fixed size.
 *
 *       the rest of the portion contains two elements which were not
 *       defined here to avoid duplication:
 *
 *       1) the device name of the logical volume which is holding the
 *          file system. 
 *
 *       2) the mount point of the file system. 
 *
 *       Since these two elements are variable length, so a file system
 *       transaction record will be also variable length.
 *
 *       The maximum length for device name or mount point is PATH_MAX.
 *       They are null terminated strings.
 */

struct tran_filesystem {
   	unsigned long long DbytesTrans; /* total bytes for read and write */
   	unsigned long long DnumRdWrs;  	/* total number of reads and writes */
   	unsigned long long DnumOpens;  	/* total number of opens */
   	unsigned long long DnumCreates; /* total number of creates */
   	unsigned long long DnumLocks;  	/* total number of locks */

   	uint Dfstype;    	/* fs type, also in vfs.vfs_mdata->vmt_gfstype 
                     		 *    its value is defined in <sys/vmount.h>
                     		 */
	short devname_len; 		/* device name length */
	short mntptr_len;   		/* mount point length */ 
};

/**********************************************************************/
/*
 * Network interface activity structure for acct record. (TRID_netif)
 */
#define TNETIF_NAMSIZ 	16
typedef struct tran_netif
{
	char		netif_name[TNETIF_NAMSIZ];
	long long	nios;
	long long	nbytes;
} tran_netif_t;

/**********************************************************************/

struct tran_disk {
        unsigned long long      total_xfers;    /* total disk transfers */
        unsigned long long      total_rblks;    /* total reads from disk */
        unsigned long long      total_wblks;    /* total writes to disk */
        unsigned int            blk_size;       /* blk size of disk transfer */
        char                    diskname[38];   /* name of disk */
};

/**********************************************************************/

struct tran_lostdata {
	int			trl_lost_recs;	/* number of lost transactions*/
	long long		trl_lost_cpu;	/* usecs lost process cputime */
	long long		trl_lost_start;	/* time data loss began, usecs*/
	long long		trl_lost_KEcpu;	/* usecs lost kernext cputime */
};

/**********************************************************************/

struct tran_vscsi_client {
        uint32_t                partition_num;  /* srvr partition number  */
        unsigned long long      unit_id;        /* srvr unit id           */
        unsigned long long      lun_id;         /* device logical unit id */
        unsigned long long      bytes_in;       /* bytes read             */
        unsigned long long      bytes_out;      /* bytes written          */
};

/**********************************************************************/

struct tran_vscsi_target {
        uint32_t                client_partition_num;  /* client number   */
        unsigned long long      unit_id;        /* srvr unit id           */
        unsigned long long      lun_id;        	/* device logical unit id */
        unsigned long long      bytes_in;      	/* bytes read             */
        unsigned long long      bytes_out;     	/* bytes written          */
};

/**********************************************************************/

/*
 * Character set mappings for ARM data.
 */
#define ACCT_ARM_ASCII		3	/* ARM_CHARSET_ASCII 		*/
#define ACCT_ARM_UTF8		106	/* ARM_CHARSET_UTF8 		*/
#define ACCT_ARM_UTF16BE	1013	/* ARM_CHARSET_UTF16BE 		*/
#define ACCT_ARM_UTF16LE	1014	/* ARM_CHARSET_UTF16LE 		*/
#define ACCT_ARM_UTF16		1015	/* ARM_CHARSET_UTF16 		*/
#define ACCT_ARM_IBM037		2028	/* ARM_CHARSET_IBM037 		*/
#define ACCT_ARM_IBM1047	2102	/* ARM_CHARSET_IBM1047 		*/

/*
 * Version numbers for data structures.
 */
#define ACCT_ARM_APPL_V1	1	/* tran_arm_applenv 		*/
#define ACCT_ARM_TRENV_V1	1	/* tran_arm_trenv 		*/
#define ACCT_ARM_TRAN_V1	1	/* tran_arm_tran 		*/
#define ACCT_ARM_AGG_V1		1	/* tran_agg_arm 		*/

/*
 * This record is produced for each unique registered application at the
 * system level.  The aeid parameter is used to refer to this unique 
 * combination of appl_name, group name, and properties.
 *
 * All offsets are calculated relative to the structure that contains them.
 * For example, ap_name is calculated relative to the start of the struct
 * tran_arm_applenv, tr_name to tran_arm_trenv, noffset to 
 * tran_arm_properties, ag_uid to tran_agg_arm, ... 
 */ 
struct tran_arm_applenv { 
	unsigned int 	   ap_version;	/* just in case			*/
	unsigned int 	   ap_charset;	/* character set for strings   	*/
	unsigned long long ap_id;	/* application environment id  	*/
	unsigned short	   ap_name;	/* offset to appl name    	*/
	unsigned short	   ap_group;	/* offset to appl group   	*/
	unsigned short	   ap_iprop;	/* offset to identity properties*/
	unsigned short	   ap_cprop;	/* offset to context properties */
	unsigned short	   ap_pad[2];
};
	
/*
 * This record is produced for each unique registered transaction at 
 * the system level.  The teid is used to refer to this unique combination
 * of transaction name and properties.  The cproperties contains only names.
 * The corresponding cproperty value fields are not provided. 
 */ 
struct tran_arm_trenv {
	unsigned int 	   tr_version;	/* just in case			*/
	unsigned int 	   tr_charset;	/* character set for strings 	*/
	unsigned long long tr_id;	/* transaction environment id	*/
	unsigned short	   tr_name;	/* offset to transaction name 	*/
	unsigned short	   tr_iprop;	/* offset to identity properties*/
	unsigned short	   tr_cprop;	/* offset to context properties */
	unsigned short	   tr_pad[3];
};

/* 
 * Arm IDs, if provided, are not unix user ids.  The content of this
 * field is defined by the application.   
 */
struct tran_arm_id {
	union {
		char	   c[16];
		int	   i[4];
		long long  l[2];
	} id_u;
};

/* 
 * Completion status for ARM transactions
 */ 
#define ACCT_ARM_STATUS_DISCARD	   -1	/* arm_discard_transaction() called */
#define ACCT_ARM_STATUS_GOOD	    0
#define ACCT_ARM_STATUS_ABORTED	    1
#define ACCT_ARM_STATUS_FAILED	    2
#define ACCT_ARM_STATUS_UNKNOWN	    3

/*
 * This record is produced for each transaction.
 */
struct tran_arm_tran {
	unsigned int 	   tri_version; /* just in case			*/
	unsigned int 	   tri_rc;	/* completion status 		*/
	unsigned long long tri_aeid;	/* application environment id  	*/
	unsigned long long tri_teid;	/* transaction environment id	*/
	unsigned short	   tri_uid;	/* offset to user identifier    */
	unsigned short	   tri_uname;	/* offset to user name 		*/
	unsigned short	   tri_acct;	/* offset to AIX_ACCT_CLASS 	*/
	unsigned short	   tri_pad2;
	unsigned int	   tri_response;/* response time (milliseconds) */
	unsigned int	   tri_queued;	/* queued time   (milliseconds)	*/
	struct tusage	   tri_util;	/* resource util (microseconds) */
};

/*
 * Context and identity properties are stored as name value pairs.  Names
 * and strings are stored as struct tran_arm_string objects.
 */
struct tran_arm_prop {
	unsigned short	   noffset;	/* name offset 			*/
	unsigned short	   voffset;	/* value offset			*/
};

#define ACCT_MAX_PROPERTIES	20	/* ARM_PROPERTY_MAX_COUNT 	*/

struct tran_arm_properties {
	unsigned short	   tp_pad;
	unsigned short	   tp_cnt;	/* number of name-value pairs */

	struct tran_arm_prop tp_nv[ACCT_MAX_PROPERTIES*2];
};

/* 
 * All ARM string data has the following format.  The character set is
 * is defined in the parent structure.
 */
struct tran_arm_string {
	unsigned short 	ts_len;		/* number of bytes in string[] */
	char		ts_string[512];	/* the actual string data */
};

/*
 * The following structure is the aggregated ARM transaction.
 */
struct tran_agg_arm {
	unsigned int 	   ag_version;	/* just in case			*/
	unsigned int	   ag_rc;	/* completion status		*/
	unsigned long long ag_start;	/* start time in seconds 	*/
	unsigned long long ag_aeid;	/* application environment id  	*/
	unsigned long long ag_teid;	/* transaction environment id	*/
	unsigned short 	   ag_uid;	/* offset to user identifier 	*/
	unsigned short	   ag_uname;	/* offset to user name 		*/
	unsigned short	   ag_acct;	/* offset to account class 	*/
	unsigned short 	   ag_pad2;

	/* statistics that are incremented */
	unsigned int	   ag_pad3;
	unsigned int 	   ag_response;	/* response time (milliseconds) */
	unsigned int 	   ag_queued;	/* queued time   (milliseconds) */
	unsigned int 	   ag_count;	/* number of records aggregated */
	struct tusage 	   ag_util;	/* resource util (microseconds)	*/
};

/**********************************************************************
 *
 * The ACCT_NAME macro below generates a code segment to be run during
 * kernel initialization.  This pre-defines certain transactions that
 * are always present in the kernel.
 *
 * This data is for kernel use only.
 */
#define TRSS	15		/* transaction string size for below */
#ifdef _KERNSYS
#define aacct_name acct_sys_data.tr_name
#endif /* _KERNSYS */
#define ACCT_NAME							\
	strncpy (aacct_name[TRID_proc],		"proc",		TRSS);	\
	strncpy (aacct_name[TRID_agg_proc],	"agg_proc",	TRSS);	\
	strncpy (aacct_name[TRID_agg_app],	"agg_app",	TRSS);	\
	strncpy (aacct_name[TRID_cpumem],	"system",	TRSS);	\
	strncpy (aacct_name[TRID_project],	"project",	TRSS);	\
	strncpy (aacct_name[TRID_policy],	"policy",	TRSS);	\
	strncpy (aacct_name[TRID_file],		"file",		TRSS);	\
	strncpy (aacct_name[TRID_netif],	"netif",	TRSS);	\
	strncpy (aacct_name[TRID_disk],		"disk",		TRSS);	\
	strncpy (aacct_name[TRID_lostdata],	"lostdata",	TRSS);	\
	strncpy (aacct_name[TRID_agg_ke],	"agg_KE",	TRSS);	\
	strncpy (aacct_name[TRID_vscsi_client],	"vscsi_client",	TRSS);	\
	strncpy (aacct_name[TRID_vscsi_target],	"vscsi_target",	TRSS);	\
	strncpy (aacct_name[TRID_arm_applenv],	"arm_applenv",	TRSS);	\
	strncpy (aacct_name[TRID_arm_trenv],	"arm_trenv",	TRSS);	\
	strncpy (aacct_name[TRID_arm_tran],	"arm_tran",	TRSS);	\
	strncpy (aacct_name[TRID_agg_arm],	"agg_arm",	TRSS);	\
	strncpy (aacct_name[TRID_pad],		"pad",		TRSS);	\
/* ACCT_NAME */

/**********************************************************************/
#endif	/* _H_AACCT */
