/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* bos520 src/bos/kernel/sys/dr.h 1.20                                    */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2001,2002          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* @(#)40     1.20  src/bos/kernel/sys/dr.h, sysproc, bos520 8/13/02 16:11:52 */

#ifndef _H_SYS_DR
#define _H_SYS_DR

#include <sys/types.h>
#include <sys/processor.h>

typedef struct dr_info {
	unsigned int        add : 1;	/* add operation		      */
	unsigned int        rem : 1; 	/* remove operation		      */
	unsigned int        cpu : 1;	/* target resource is a cpu	      */
	unsigned int        mem : 1;	/* target resource is memory	      */
	unsigned int      check : 1;	/* check phase in effect	      */
	unsigned int        pre : 1;	/* pre phase in effect		      */
	unsigned int       doit : 1;	/* doit phase in effect		      */
	unsigned int       post : 1;	/* post phase in effect		      */
	unsigned int  posterror : 1;	/* post error phase in effect	      */
	unsigned int      force : 1;	/* force option in effect	      */
	unsigned int   bindproc : 1;	/* process has bindprocess dependency */
	unsigned int   softpset : 1;	/* process has soft processor set
					   dependency                         */
	unsigned int   hardpset : 1;	/* process has hard processor set
					   dependency			      */
	unsigned int      plock : 1;	/* process has plock'd memory         */
	unsigned int       pshm : 1;	/* process has pinned shared memory   */

	/* The following fields are filled out for cpu based requests */

	int	lcpu;			/* logical id of target cpu 	      */
	int	bcpu;			/* bind-id of target cpu	      */

	/* The following fields are filled out for memory based requests */

        size64_t  req_memsz_change;	/* Request size in bytes              */
	size64_t  sys_memsz;		/* System Memory size		      */
        rpn64_t   sys_free_frames;	/* Number of free frames in the system*/
        rpn64_t   sys_pinnable_frames;	/* Number of pinnable frames in system*/
        rpn64_t   sys_total_frames;	/* Total number of frames in system   */

	int  reserved[12];          
} dr_info_t;

int dr_reconfig(int flags, dr_info_t *info);

/* Valid flags for dr_reconfig are */
#define DR_QUERY            0x000000001
#define DR_EVENT_FAIL       0x000000002

#ifdef _KERNEL
/* 
 * DR Events that Kernel Extensions wants to be notified of 
 */ 
#define DR_CPU_ADD_CHECK		0x00000001
#define DR_CPU_ADD_PRE			0x00000002
#define DR_CPU_ADD_POST			0x00000004
#define DR_CPU_ADD_POST_ERROR		0x00000008
#define DR_CPU_REMOVE_CHECK		0x00000010
#define DR_CPU_REMOVE_PRE		0x00000020
#define DR_CPU_REMOVE_POST		0x00000040
#define DR_CPU_REMOVE_POST_ERROR	0x00000080
#define DR_MEM_ADD_CHECK		0x00000100
#define DR_MEM_ADD_PRE			0x00000200
#define DR_MEM_ADD_POST			0x00000400
#define DR_MEM_ADD_POST_ERROR		0x00000800
#define DR_MEM_REMOVE_CHECK		0x00001000
#define DR_MEM_REMOVE_PRE		0x00002000
#define DR_MEM_REMOVE_POST		0x00004000
#define DR_MEM_REMOVE_POST_ERROR	0x00008000


/*
 * Events flag values grouped for ease.
 */
#define DR_CHECK_NOTIFY	(DR_MEM_REMOVE_CHECK | DR_CPU_REMOVE_CHECK | \
			 DR_CPU_ADD_CHECK | DR_MEM_ADD_CHECK)

#define DR_PRE_NOTIFY	(DR_MEM_REMOVE_PRE | DR_CPU_REMOVE_PRE | \
			 DR_CPU_ADD_PRE | DR_MEM_ADD_PRE)

#define DR_POST_NOTIFY	(DR_MEM_REMOVE_POST | DR_CPU_REMOVE_POST | \
			 DR_CPU_ADD_POST | DR_MEM_ADD_POST)

#define DR_POST_ERROR_NOTIFY	(DR_MEM_REMOVE_POST_ERROR | \
				 DR_CPU_REMOVE_POST_ERROR | \
				 DR_CPU_ADD_POST_ERROR    | \
				 DR_MEM_ADD_POST_ERROR)

#define DR_CPU_NOTIFY 	(DR_CPU_ADD_CHECK    | DR_CPU_ADD_PRE        | \
			 DR_CPU_ADD_POST     | DR_CPU_ADD_POST_ERROR | \
			 DR_CPU_REMOVE_CHECK | DR_CPU_REMOVE_PRE     | \
			 DR_CPU_REMOVE_POST  | DR_CPU_REMOVE_POST_ERROR)

#define DR_MEM_NOTIFY 	(DR_MEM_ADD_CHECK    | DR_MEM_ADD_PRE        | \
			 DR_MEM_ADD_POST     | DR_MEM_ADD_POST_ERROR | \
			 DR_MEM_REMOVE_CHECK | DR_MEM_REMOVE_PRE     | \
			 DR_MEM_REMOVE_POST  | DR_MEM_REMOVE_POST_ERROR)

#define DR_ALL_NOTIFY	(DR_CPU_NOTIFY | DR_MEM_NOTIFY)

/* 
 * DR handler return values: 
 */
#define DR_SUCCESS  0
#define DR_WAIT     1
#define DR_FAIL     2

/*
 * Output data structures to kernel extensions for cpu and memory
 * DR operations
 */

struct dri_cpu {
	cpu_t		lcpu;		/* Logical CPU Id of target CPU */
	cpu_t		bcpu;		/* Bind Id being removed	*/
};

struct dri_mem {
	size64_t	req_memsz_change;   /* requested mem size  */
	size64_t	sys_memsz;	    /* Original total mem size */
	size64_t	act_memsz_change;   /* mem added/removed so far */
	rpn64_t	        sys_free_frames;    /* Number of free frames */
	rpn64_t	        sys_pinnable_frames;/* Number of pinnable frames */
	rpn64_t	        sys_total_frames;   /* Total number of frames */
	unsigned long long lmb_addr;   	    /* start addr of LMB */
	size64_t	lmb_size;	    /* Size of LMB being added */
};

extern int	reconfig_register( int (*)(void *, void *, int, void *),
				   int, void * , ulong *, char *);
extern int	reconfig_unregister(ulong);
extern void	reconfig_complete(void *, int);
extern void	dr_mem_sync(void);


#endif /* _KERNEL */

#endif /* _H_SYS_DR */
